import uuid
import socket
import time
import requests
import subprocess
import os
import sys


SERVER_API_KEY = '123'
BASE_DIR = os.path.abspath(os.path.dirname(__file__))
BASE_DIR = BASE_DIR.replace('\\', '/')

try:
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(('8.8.8.8', 80))
    g_current_ip = s.getsockname()[0]
    g_current_ip = '127.0.0.1'
    print(g_current_ip)
except Exception as e:
    s.close()
    print(e)
    raise ValueError('failed to get VPS IPv4')


def MyRequest(url, dicts):
    try:
        global SERVER_API_KEY
        dicts['api_key'] = SERVER_API_KEY
        resp = requests.get(url, params=dicts, timeout=10)
        return resp.text
    except Exception as e:
        print(e)
    return ""


def RunCMD(cmd, is_wait=True):
    print('=' * 10)
    print('is_wait: %s $: %s' % (is_wait, cmd))
    print('↓' * 10)
    sys.stdout.flush()
    if is_wait:
        subprocess.Popen(cmd, shell=True).wait()
    else:
        subprocess.Popen(cmd, shell=True)


#install deps
RunCMD('python3 -m pip install psutil')
RunCMD('python3 -m pip install aiohttp')
RunCMD('python3 -m pip install requests')
RunCMD('python3 -m pip install ntplib')

import install

# run server and client
RunCMD('python3 %s' % (BASE_DIR + '/main.py'), is_wait=False)
RunCMD('python3 %s' % (BASE_DIR + '/server/server_main.py'), is_wait=False)

time.sleep(20)  # wait them to be ok.

# check UserLogin function
my_uuid = uuid.uuid1()
url = 'http://%s:8082' % (g_current_ip)

print('UserLogin ', MyRequest(url, {'type': 'UserLogin', 'user_key': my_uuid}))

# check DisConnect function
print('DisConnect ', MyRequest(url, {
    'type': 'DisConnect',
    'user_key': my_uuid
}))

RunCMD('python3 %s --stop' % (BASE_DIR + '/main.py'))



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import sys
import requests
import argparse
import time
import socket
import threading
import json
import subprocess
from aiohttp import web

try:
    from . import install
except:
    import install

parser = argparse.ArgumentParser(
    description='EasyCompleteYou, Easily complete you.')
parser.add_argument('--stop', action='store_true', help='stop everything')

parser.add_argument('--is_use_cloudfare',
                    action='store_true',
                    help='stop everything')

parser.add_argument('--domain', help='domain for server')
g_args = parser.parse_args()


def Get(dicts, key, default_value=""):
    if key in dicts:
        return dicts[key]
    return default_value


def GetCurrentOS():
    temp = sys.platform
    if temp == 'win32':
        return 'Windows'
    if temp == 'cygwin':
        return 'Cygwin'
    if temp == 'darwin':
        return 'Mac'
    return 'Linux'


def AllowPort(port):
    if GetCurrentOS() == 'Linux':
        subprocess.Popen('ufw allow %s' % (port), shell=True).wait()


def DeletePort(port):
    if GetCurrentOS() == 'Linux':
        subprocess.Popen('ufw delete allow %s' % (port), shell=True).wait()


SERVER_API_KEY = "123"


def MyRequest(url, dicts, is_return=True):
    try:
        global SERVER_API_KEY
        dicts['api_key'] = SERVER_API_KEY
        resp = requests.get(url, params=dicts, timeout=10)
        return resp.text
    except Exception as e:
        print(e)
    return ""


def Stop():
    try:
        global SERVER_API_KEY
        dicts['api_key'] = SERVER_API_KEY
        resp = requests.get(url, params=dicts, timeout=10)
        return resp.text
    except Exception as e:
        print(e)
    return ""


class ClassName(object):
    def __init__(self, CLIENT_API_PORT="8082", exposes_api=False):
        self.SERVER_API_UIL = "http://uifv2ray.xyz:8081/api"
        self.SERVER_API_UIL = "http://127.0.0.1:8081/api"  # for test

        self.CLIENT_API_PORT = CLIENT_API_PORT
        self.GO_REMOVE_USER_URL = "http://127.0.0.1:9090/removeUser"
        self.GO_ADD_USER_URL = "http://127.0.0.1:9090/addUser"
        self.GO_CLOSE_URL = "http://127.0.0.1:9090/quit"
        global SERVER_API_KEY
        self.SERVER_API_KEY = SERVER_API_KEY
        self.exposes_api = exposes_api
        self.g_current_ip = ''
        self.ClientInfo()  # init self.g_current_ip

        self.g_user_list = {}

    def CloseClient(self):
        MyRequest(self.ClientInfo(), {'type': 'Quit'})
        print('closed')
        quit()

    def Start(self):
        global g_args

        DeletePort(9090)  # my v2ray grpc port
        DeletePort(10085)  # v2ray grpc original prot

        AllowPort(self.CLIENT_API_PORT)  # client
        AllowPort(80)
        AllowPort(443)

        if GetCurrentOS() == "Linux":
            subprocess.Popen('fuser -k -n tcp 80', shell=True).wait()

        if g_args.domain is None:
            print('Have no domain. Netword can not be detected in public.')

        self.bg = install.Caddy_TLS_WS_VMESS(
            g_args.domain,
            use_cloudfare=g_args.is_use_cloudfare)  # only with domain

        app = web.Application()
        threading.Thread(target=self._heart_beat).start()
        app.add_routes([web.get('/', self.Dispash)])
        web.run_app(app,
                    host=self.g_current_ip,
                    port=int(self.CLIENT_API_PORT))  # block here

    def Die(self):
        install.Die()
        self.CloseControler()
        quit()

    async def Dispash(self, req):  # Request from user
        params = req.rel_url.query
        user_key = Get(params, 'user_key')
        api_key = Get(params, 'api_key')
        if user_key == '':
            return web.Response(text="fuck")
        types = Get(params, 'type')
        text = "wrong type"
        if types == 'UserLogin':  # from users
            text = self.UserLogin(params, user_key)
        elif types == 'DisConnect' and api_key == self.SERVER_API_KEY:  # from server
            text = self.DisConnect(user_key)
        elif types == 'Quit' and api_key == self.SERVER_API_KEY:
            self.Die()
        elif types == 'CheckAlive':  # from server
            text = {'is_ok': True}

        if type(text) is dict:
            text = json.dumps(text)
        return web.Response(text=text)

    def ClientInfo(self):  # myself info
        if not self.exposes_api:
            self.g_current_ip = '127.0.0.1'
        if self.g_current_ip == '':
            # get IPv4
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('8.8.8.8', 80))
                self.g_current_ip = s.getsockname()[0]
            except Exception as e:
                s.close()
                print(e)
                raise ValueError('failed to get VPS IPv4')
        return "http://%s:%s/api" % (self.g_current_ip, self.CLIENT_API_PORT)

    def _heart_beat(self):
        while True:
            time.sleep(20 * 60)  # 20 minutes
            for item in self.g_user_list:
                MyRequest(
                    self.SERVER_API_UIL, {
                        'user_key': item,
                        'type': 'Update',
                        'used_flow': self.ReadUserFlow(item)
                    })

    def ReadUserFlow(self, user_key):
        try:
            return "1"
        except Exception as e:
            print(e)
            return ""

    def DisConnect(self, user_key):  # remove user
        if user_key not in self.g_user_list:
            return
        del self.g_user_list[user_key]
        res = MyRequest(self.GO_REMOVE_USER_URL, {'user_key': user_key})
        return res

    def _add_user(self, user_key):  # can only call by center's server
        t = time.time()
        t = int(t)
        if user_key not in self.g_user_list:
            self.g_user_list[user_key] = {}
        self.g_user_list[user_key]['last_time'] = t
        print(MyRequest(self.GO_ADD_USER_URL, {'user_key': user_key}))

    def CloseControler(self):
        print(MyRequest(self.GO_CLOSE_URL, {}))

    def UserLogin(self, params, user_key):
        payload = {
            'type': 'UserLogin',
            'user_key': user_key,
            'client_address': self.ClientInfo()
        }
        res = ""
        try:
            res = MyRequest(self.SERVER_API_UIL, payload)
            temp = json.loads(res)
            if temp['status'] == 0:  # check by server
                self._add_user(user_key)
            else:
                self.DisConnect(user_key)

            # for safety reason
            # try:
            #     del temp['params']['using']
            # except:
            #     pass
            # res = json.dumps(temp)
        except Exception as e:
            print(e)
        return res


def Main():
    temp = ClassName()
    if g_args.stop:
        temp.CloseClient()
    else:
        temp.Start()
        temp.CloseClient()


if __name__ == '__main__':
    Main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import sys
import requests
import argparse
import socket
import os
import subprocess
import time
import ntplib

try:
    import psutil
except:
    subprocess.Popen('python3 -m pip install psutil', shell=True).wait()
    import psutil

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
BASE_DIR = BASE_DIR.replace('\\', '/')

parser = argparse.ArgumentParser(
    description='EasyCompleteYou, Easily complete you.')
parser.add_argument('--stop', action='store_true', help='stop everything')

parser.add_argument('--is_use_cloudfare',
                    action='store_true',
                    help='stop everything')

parser.add_argument('--domain', help='domain for server')
g_args = parser.parse_args()


def IsProcessExist(name):
    pids = psutil.pids()
    for pid in pids:
        try:
            p = psutil.Process(pid)
            temp = p.name()
            if name == temp:
                return True
        except Exception as e:
            pass
    print("process '%s' is not exist." % (name))
    return False


SERVER_API_KEY = "123"


def MyRequest(url, dicts):
    try:
        global SERVER_API_KEY
        dicts['api_key'] = SERVER_API_KEY
        resp = requests.get(url, params=dicts, timeout=10)
        return resp.text
    except Exception as e:
        print(e)
    return ""

def AjustTime():
    c = ntplib.NTPClient()
    response = c.request('pool.ntp.org')
    ts = response.tx_time

    _date = time.strftime('%Y-%m-%d', time.localtime(ts))
    _time = time.strftime('%X', time.localtime(ts))
    os.system('date {} && time {}'.format(_date, _time))


def GetCurrentOS():
    temp = sys.platform
    if temp == 'win32':
        return 'Windows'
    if temp == 'cygwin':
        return 'Cygwin'
    if temp == 'darwin':
        return 'Mac'
    return 'Linux'


if GetCurrentOS() == 'Linux':
    LINUX_CADDY_PATH = BASE_DIR + '/caddy/caddy_2.3.0_linux_amd64/caddy'
    LINUX_V2RAY_PATH = BASE_DIR + '/v2ray/linux/v2ray-linux-64/v2ray'
    LINUX_CONTR_PATH = BASE_DIR + '/controler/contro_linux'
else:
    LINUX_CADDY_PATH = BASE_DIR + '/caddy/caddy_2.3.0_windows_amd64/caddy.exe'
    LINUX_V2RAY_PATH = BASE_DIR + '/v2ray/windows/v2ray-windows-64/v2ray.exe'
    LINUX_CONTR_PATH = BASE_DIR + '/controler/contro_windows.exe'


def GetPermission():
    if GetCurrentOS() == 'Linux':
        cmd = "sudo chmod -R 750 " + BASE_DIR
        subprocess.Popen(cmd, shell=True).wait()


def RunV2ray(argv=""):
    cmd = LINUX_V2RAY_PATH + ' ' + argv
    if GetCurrentOS == 'Linux':
        cmd = 'sudo nohup %s' % (cmd)
    print(cmd)
    subprocess.Popen(cmd, shell=True)
    RunControler()
    print('v2ray is running.')


def Die():
    KillCaddy()
    KillV2ray()


def KillV2ray():
    cmd = LINUX_V2RAY_PATH + ' stop'
    subprocess.Popen(cmd, shell=True)
    print('v2ray dead')


def RunControler(argv=""):
    cmd = LINUX_CONTR_PATH + ' ' + argv
    if GetCurrentOS == 'Linux':
        cmd = 'sudo nohup %s' % (cmd)
    print(cmd)
    subprocess.Popen(cmd, shell=True)
    print('controler is running.')


def RunCaddy(argv=""):
    cmd = LINUX_CADDY_PATH + ' ' + argv
    if GetCurrentOS == 'Linux':
        cmd = 'sudo nohup %s' % (cmd)
    print(cmd)
    subprocess.Popen(cmd, shell=True)
    print('caddy is running.')


def KillCaddy():
    cmd = LINUX_CADDY_PATH + ' stop'
    subprocess.Popen(cmd, shell=True)
    print('caddy dead')


def GetMyIP(is_ipv6=False):
    if is_ipv6:
        return MyRequest('https://v6.ident.me/', {})
    return MyRequest('https://v4.ident.me/', {})


def CheckPort(port):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('localhost', port)) == 0


class V2ray_And_Caddy():
    def __init__(self, v2ray_config, caddy_config):
        self.v2ray_config = v2ray_config
        self.caddy_config = caddy_config
        self.caddy_test_address = 'http://127.0.0.1:3566'
        if CheckPort(80) or CheckPort(443):
            raise ValueError("port 80 or 443 is in use.")

        RunCaddy(argv='run -config "%s" -adapter caddyfile' %
                 (self.caddy_config))
        RunV2ray(argv='-config "%s"' % (self.v2ray_config))
        time.sleep(10)

    def Die(self):
        KillCaddy()
        KillV2ray()

    def TestCaddyOK(self):
        if GetCurrentOS() != 'Linux':
            return True
        if MyRequest(self.caddy_test_address, {}) != 'ok':
            return False
        return True


class Caddy_TLS_WS_VMESS:
    """ domain is needed
    """
    def __init__(self, domain=None, is_ipv6=True, use_cloudfare=False):
        self.is_init = False
        if CheckPort(80) or CheckPort(443):
            raise ValueError("port 80 or 443 is in use.")

        self._init_domain(domain, use_cloudfare)
        self.InitV2rayConfig(is_ipv6)

        self.InitCaddyConfig()

        AjustTime()

        # run
        self.instance = V2ray_And_Caddy(self.v2ray_config_file_path,
                                        self.caddy_config_file_path)
        self.is_init = True

    def InitV2rayConfig(self, is_ipv6):
        # v2ray
        if is_ipv6:
            self.v2ray_config_file_path = BASE_DIR + '/v2ray_server_configs/vmess_tls_ws_ipv6.json'
        else:
            self.v2ray_config_file_path = BASE_DIR + '/v2ray_server_configs/vmess_tls_ws.json'
        self.v2ray_config_file_path = BASE_DIR + '/v2ray_server_configs/vmess_tls_ws_free.json'

    def _init_domain(self, domain, is_use_cloudfare):
        self.is_use_cloudfare = is_use_cloudfare
        if domain is not None:
            self.domain = domain
            if not self.is_use_cloudfare:  # test dns ip
                domain_ip = socket.gethostbyname(self.domain)
                my_ip = GetMyIP()
                if my_ip != domain_ip:
                    print(my_ip, domain_ip)
                    raise ValueError('wrong dns ip')
        else:
            self.domain = 'localhost'

    def Del(self):
        if self.is_init is False:
            return
        self.instance.Die()
        print('已关闭 %s' % (self.domain))

    def InitCaddyConfig(self):

        self.caddy_config_file_path = BASE_DIR + '/caddy_config/tls_ws.caddyfile'
        self.caddy_template_config_file_path = BASE_DIR + '/caddy_config/tls_ws.template'

        with open(self.v2ray_config_file_path, encoding='utf-8') as f:
            content = f.read()
        print('v2ray config', content)

        with open(self.caddy_template_config_file_path,
                  encoding='utf-8') as caddy:
            content = caddy.read()

        content = content.replace('{domain}', self.domain)
        print('caddy config', content)

        with open(self.caddy_config_file_path, 'w', encoding='utf-8') as caddy:
            caddy.write(content)


GetPermission()  # we need to

if __name__ == '__main__':
    if g_args.stop:
        Die()
    elif g_args.domain is None:
        temp = Caddy_TLS_WS_VMESS()
    else:
        temp = Caddy_TLS_WS_VMESS(g_args.domain,
                                  use_cloudfare=g_args.is_use_cloudfare)
